@@ -74,7 +74,7 @@ grab_key_real (guint      keycode,
                int        mask)
 {
         if (grab) {
-                XGrabKey (GDK_DISPLAY (),
+                XGrabKey (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                           keycode,
                           mask,
                           GDK_WINDOW_XID (root),
@@ -82,7 +82,7 @@ grab_key_real (guint      keycode,
                           GrabModeAsync,
                           GrabModeAsync);
         } else {
-                XUngrabKey (GDK_DISPLAY (),
+                XUngrabKey (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                             keycode,
                             mask,
                             GDK_WINDOW_XID (root));
@@ -219,7 +219,7 @@ match_key (Key *key, XEvent *event)
 		group = XkbGroupForCoreState (event->xkey.state);
 	else
 #endif
-		group = (event->xkey.state & GDK_Mode_switch) ? 1 : 0;
+		group = (event->xkey.state & GDK_KEY_Mode_switch) ? 1 : 0;
 
 	/* Check if we find a keysym that matches our current state */
 	if (gdk_keymap_translate_keyboard_state (NULL, event->xkey.keycode,
