@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  * gnome-settings-keyboard-xkb.h
  *
- * Copyright  2001 Udaltsoft
+ * Copyright (C) 2001 Udaltsoft
  *
  * Written by Sergey V. Oudaltsov <svu@users.sourceforge.net>
  *
@@ -28,9 +28,10 @@
 #include <gconf/gconf-client.h>
 
 #include <libxklavier/xklavier.h>
+#include "gsd-keyboard-manager.h"
 
-void gsd_keyboard_xkb_init (GConfClient * client);
-void gsd_keyboard_xkb_load (GConfClient * client);
+void gsd_keyboard_xkb_init (GsdKeyboardManager *manager);
+void gsd_keyboard_xkb_shutdown (void);
 
 typedef void (*PostActivationCallback) (void *userData);
 
@@ -38,8 +39,4 @@ void
 gsd_keyboard_xkb_set_post_activation_callback (PostActivationCallback fun,
                                                void                  *userData);
 
-GType keyboard_config_registry_get_type (void);
-
-extern XklEngine *xkl_engine;
-
 #endif
